@@ -1,45 +1,8 @@
 import os
 
 from cirrocumulus.anndata_dataset import AnndataDataset
+from cirrocumulus.io_util import get_markers, filter_markers, add_spatial, SPATIAL_HELP
 from cirrocumulus.parquet_dataset import ParquetDataset
-from cirrocumulus.simple_data import SimpleData
-
-
-def get_cell_type_genes(cell_type):
-    all_genes = []
-
-    for cell_type_markers in cell_type['markers']:
-        all_genes += cell_type_markers['genes']
-
-    for i in range(len(all_genes)):
-        gene = all_genes[i]
-        last_char = gene[len(gene) - 1]
-        if last_char == '+' or last_char == '-':
-            all_genes[i] = gene[:len(gene) - 1]
-    return all_genes
-
-
-def get_markers(marker_paths):
-    marker_dict = {}
-    import json
-
-    for marker_path in marker_paths:
-        markers = {}
-        with open(marker_path, 'rt') as f:
-            marker_json = json.load(f)
-            if 'title' in marker_json:
-                marker_dict[marker_json['title']] = markers
-                for cell_type in marker_json['cell_types']:
-                    markers[cell_type['name']] = get_cell_type_genes(cell_type)
-                    if 'subtypes' in cell_type:
-                        for cell_sub_type in cell_type['subtypes']['cell_types']:
-                            markers[cell_sub_type['name']] = get_cell_type_genes(cell_sub_type)
-            else:
-                key = os.path.splitext(os.path.basename(marker_path))[0]
-                markers = marker_dict.get(key, {})
-                marker_dict[key] = markers
-                markers.update(marker_json)
-    return marker_dict
 
 
 def configure(list_of_dataset_paths, spatial_directories, backed, marker_paths):
@@ -110,7 +73,8 @@ def configure(list_of_dataset_paths, spatial_directories, backed, marker_paths):
             spatial_directory = spatial_directories[i]
             if spatial_directory != '':
                 adata = anndata_dataset.get_data(dataset_ids[i])
-                SimpleData.add_spatial(adata, spatial_directory)
+                if not add_spatial(adata, spatial_directory):
+                    print('No spatial data found in {}'.format(spatial_directory))
 
     if marker_paths is not None and len(marker_paths) > 0:
         marker_dict = get_markers(marker_paths)
@@ -118,7 +82,8 @@ def configure(list_of_dataset_paths, spatial_directories, backed, marker_paths):
             d = anndata_dataset.get_data(dataset_id)
             existing_marker_dict = d.uns.get('markers', {})
             existing_marker_dict.update(marker_dict)
-            d.uns['markers'] = existing_marker_dict
+            # remove genes in dict that are not in dataset
+            d.uns['markers'] = filter_markers(d, existing_marker_dict)
 
 
 def create_app():
@@ -150,8 +115,7 @@ def main(argsv):
         nargs='*')
     parser.add_argument('--port', help='Server port', default=5000, type=int)
     parser.add_argument('--no-open', dest='no_open', help='Do not open your web browser', action='store_true')
-    parser.add_argument('--spatial', help='Directory containing spatial data (images, scaling factors, positions)',
-        nargs='*')
+    parser.add_argument('--spatial', help=SPATIAL_HELP, nargs='*')
 
     args = parser.parse_args(argsv)
     app = create_app()
