@@ -8,6 +8,7 @@ import Tab from '@material-ui/core/Tab';
 import Tabs from '@material-ui/core/Tabs';
 import Toolbar from '@material-ui/core/Toolbar';
 import AccountCircle from '@material-ui/icons/AccountCircle';
+import Brightness2Icon from '@material-ui/icons/Brightness3';
 import HelpIcon from '@material-ui/icons/Help';
 import MoreVertIcon from '@material-ui/icons/MoreVert';
 import React from 'react';
@@ -19,14 +20,26 @@ import {
     IMPORT_DATASET_DIALOG,
     login,
     logout,
+    setChartOptions,
     setDataset,
     setDialog,
     setMessage,
+    setSavedDatasetState,
     setTab,
 } from './actions';
 import {drawerWidth} from './App';
 import {intFormat} from './formatters';
-import {DEFAULT_INTERPOLATOR, DEFAULT_MARKER_OPACITY, DEFAULT_UNSELECTED_MARKER_OPACITY} from "./reducers";
+import {
+    DEFAULT_DARK_MODE,
+    DEFAULT_INTERPOLATOR,
+    DEFAULT_LABEL_FONT_SIZE,
+    DEFAULT_LABEL_STROKE_WIDTH,
+    DEFAULT_MARKER_OPACITY,
+    DEFAULT_SHOW_AXIS,
+    DEFAULT_SHOW_FOG,
+    DEFAULT_SHOW_LABELS,
+    DEFAULT_UNSELECTED_MARKER_OPACITY
+} from "./reducers";
 
 
 const styles = theme => ({
@@ -122,24 +135,50 @@ class AppHeader extends React.PureComponent {
         this.setState({moreMenuOpen: true, moreMenuAnchorEl: event.currentTarget});
     };
 
-    copyLink = (event) => {
-
-        const {dataset, embeddings, searchTokens, datasetFilter, interpolator, markerOpacity, unselectedMarkerOpacity, dotPlotData} = this.props;
-        let linkText = window.location.protocol + '//' + window.location.host;
+    getLinkJson = () => {
+        const {chartOptions, primaryTraceKey, dataset, embeddings, searchTokens, datasetFilter, interpolator, markerOpacity, unselectedMarkerOpacity, dotPlotData} = this.props;
 
         let json = {
             dataset: dataset.id,
-            embeddings: embeddings.map(embedding=>{
-                if(embedding.bin) {
+            embeddings: embeddings.map(embedding => {
+                if (embedding.bin) {
                     embedding = Object.assign({}, embedding);
                     delete embedding._bin;
                     return embedding;
-                }else {
-                    return {name:embedding.name, dimensions:embedding.dimensions}
+                } else {
+                    return {name: embedding.name, dimensions: embedding.dimensions};
                 }
 
             })
         };
+
+        const chartRef = chartOptions.ref;
+        if (json.embeddings.length > 0 && chartRef != null && chartRef.scatterPlot) {
+            json.camera = chartRef.scatterPlot.getCameraDef();
+
+        }
+
+        let jsonChartOptions = {};
+        if (json.embeddings.length > 0) {
+            jsonChartOptions.activeEmbedding = primaryTraceKey;
+        }
+        const defaultChartOptions = {
+            showLabels: DEFAULT_SHOW_LABELS, showAxis: DEFAULT_SHOW_AXIS,
+            showFog: DEFAULT_SHOW_FOG, darkMode: DEFAULT_DARK_MODE,
+            labelFontSize: DEFAULT_LABEL_FONT_SIZE,
+            labelStrokeWidth: DEFAULT_LABEL_STROKE_WIDTH
+        };
+
+        for (let key in defaultChartOptions) {
+            let value = chartOptions[key];
+            if (value !== defaultChartOptions[key]) {
+                jsonChartOptions[key] = value;
+            }
+        }
+        if (Object.keys(jsonChartOptions).length > 0) {
+            json.chartOptions = jsonChartOptions;
+        }
+
         if (searchTokens.length > 0) {
             json.q = searchTokens;
         }
@@ -176,7 +215,13 @@ class AppHeader extends React.PureComponent {
         if (interpolator.name !== DEFAULT_INTERPOLATOR) {
             json.colorScheme = interpolator.name;
         }
-        linkText += '?q=' + JSON.stringify(json);
+        return json;
+    };
+
+    copyLink = (event) => {
+
+        let linkText = window.location.protocol + '//' + window.location.host;
+        linkText += '?q=' + JSON.stringify(this.getLinkJson());
         const container = document.activeElement;
         const fakeElem = document.createElement('textarea');
         const isRTL = document.documentElement.getAttribute('dir') == 'rtl';
@@ -212,6 +257,11 @@ class AppHeader extends React.PureComponent {
     };
 
 
+    onDarkMode = () => {
+        this.props.chartOptions.darkMode = !this.props.chartOptions.darkMode;
+        this.props.handleChartOptions(this.props.chartOptions);
+    };
+
     handleLogout = () => {
         this.setState({userMenuOpen: false});
         this.props.handleLogout();
@@ -222,6 +272,13 @@ class AppHeader extends React.PureComponent {
         this.setState({moreMenuOpen: false});
     };
     handleDataset = (event) => {
+        if (this.props.dataset != null) {
+            const savedDatasetState = this.props.savedDatasetState;
+            const link = this.getLinkJson();
+            link.dataset = null;
+            savedDatasetState[this.props.dataset.id] = link;
+            this.props.handleSavedDatasetState(savedDatasetState);
+        }
         this.props.handleDataset(event.target.value);
     };
     handleSettings = (event) => {
@@ -322,24 +379,31 @@ class AppHeader extends React.PureComponent {
 
                             {showEditDeleteDataset && <MenuItem onClick={this.handleSettings}>Edit Dataset</MenuItem>}
                             {showEditDeleteDataset && <MenuItem onClick={this.handleDelete}>Delete Dataset</MenuItem>}
-                            {(showNewDataset || showEditDeleteDataset) && dataset!=null && <Divider/>}
+                            {(showNewDataset || showEditDeleteDataset) && dataset != null && <Divider/>}
                             {dataset != null && <MenuItem onClick={this.copyLink}>Copy Link </MenuItem>}
                         </Menu>}
+
+
+                        {<Tooltip title={"Toggle Light/Dark Theme"}>
+                            <IconButton edge={false} className={this.props.chartOptions.darkMode ? 'cirro-active' : ''}
+                                        aria-label="Toggle Theme" onClick={() => this.onDarkMode()}>
+                                <Brightness2Icon/>
+                            </IconButton>
+                        </Tooltip>}
                         <Tooltip title={'Help'}>
                             <IconButton aria-label="Help"
                                         onClick={this.handleHelp}>
                                 <HelpIcon/>
                             </IconButton>
                         </Tooltip>
-
-                        {email != null && email!==''&&
+                        {email != null && email !== '' &&
                         <Tooltip title={email}>
                             <IconButton aria-label="Menu" aria-haspopup="true"
                                         onClick={this.handleUserMenuOpen}>
                                 <AccountCircle/>
                             </IconButton>
                         </Tooltip>}
-                        {email != null && email!==''&&
+                        {email != null && email !== '' &&
                         <Menu id="menu-user"
                               anchorEl={this.state.userMenuAnchorEl}
                               anchorOrigin={{
@@ -370,16 +434,17 @@ class AppHeader extends React.PureComponent {
 const mapStateToProps = state => {
     return {
         dataset: state.dataset,
-
+        chartOptions: state.chartOptions,
         embeddings: state.embeddings,
         searchTokens: state.searchTokens,
+        primaryTraceKey: state.primaryTraceKey,
         binSummary: state.binSummary,
         binValues: state.binValues,
         combineDatasetFilters: state.combineDatasetFilters,
         datasetFilter: state.datasetFilter,
         markerOpacity: state.markerOpacity,
         unselectedMarkerOpacity: state.unselectedMarkerOpacity,
-
+        savedDatasetState: state.savedDatasetState,
         datasetChoices: state.datasetChoices,
         dialog: state.dialog,
         dotPlotData: state.dotPlotData,
@@ -411,11 +476,19 @@ const mapDispatchToProps = (dispatch, ownProps) => {
         handleLogout: () => {
             dispatch(logout());
         },
+
+        handleSavedDatasetState: value => {
+            dispatch(setSavedDatasetState(value));
+        },
+
         handleDataset: value => {
             dispatch(setDataset(value));
         },
         handleDialog: (value) => {
             dispatch(setDialog(value));
+        },
+        handleChartOptions: (value) => {
+            dispatch(setChartOptions(value));
         }
     };
 };
