@@ -8,12 +8,11 @@ import MenuItem from '@material-ui/core/MenuItem';
 import Paper from '@material-ui/core/Paper';
 import Select from '@material-ui/core/Select';
 import PhotoCameraIcon from '@material-ui/icons/PhotoCamera';
-import {scaleLinear, scaleSequential} from 'd3-scale';
-import {interpolateReds} from 'd3-scale-chromatic';
+import natsort from 'natsort';
 import React from 'react';
-import ColorSchemeLegend, {drawColorScheme} from './ColorSchemeLegend';
+import {drawColorScheme} from './ColorSchemeLegend';
 import {numberFormat} from './formatters';
-import SizeLegend, {drawSizeLegend} from './SizeLegend';
+import {drawSizeLegend} from './SizeLegend';
 
 const styles = theme => ({
     root: {},
@@ -26,8 +25,7 @@ const styles = theme => ({
 
 let svgFont = '12px Helvetica,Arial,sans-serif';
 let canvasFont = '12px Roboto Condensed,Helvetica,Arial,sans-serif';
-let maxRadius = 9;
-let minRadius = 1;
+
 let gridColor = '#808080';
 let gridThickness = 0.5;
 
@@ -59,6 +57,7 @@ class DotPlotCanvas extends React.PureComponent {
         if (this.canvas == null) {
             let onMouseMove = (event) => {
                 const node = event.target;
+                const maxRadius = this.props.sizeScale.range()[1];
                 var rect = node.getBoundingClientRect();
                 let xy = [event.clientX - rect.left - node.clientLeft, event.clientY - rect.top - node.clientTop];
                 // xy[0] /= devicePixelRatio;
@@ -106,11 +105,12 @@ class DotPlotCanvas extends React.PureComponent {
     drawContext(context) {
         const renamedCategories = this.props.renamedCategories || {};
         const dotplot = this.dotplot;
-        const colorScale = this.colorScale;
+        const colorScale = this.props.colorScale;
         const features = this.features;
-        const sizeScale = this.sizeScale;
+        const sizeScale = this.props.sizeScale;
         const categories = this.categories;
         const categoryOrder = this.categoryOrder;
+        const maxRadius = sizeScale.range()[1];
         let diameter = maxRadius * 2;
         // context.strokeStyle = gridColor;
         // context.lineWidth = gridThickness;
@@ -203,6 +203,7 @@ class DotPlotCanvas extends React.PureComponent {
             xoffset = Math.max(xoffset, context.measureText(category).width);
         });
         xoffset += 4;
+        const maxRadius = this.props.sizeScale.range()[1];
         const diameter = maxRadius * 2;
         const height = this.categories.length * diameter + 4;
         const width = this.features.length * diameter + 4;
@@ -253,6 +254,7 @@ class DotPlotCanvas extends React.PureComponent {
             }
         } else { // sort by category
             if (Object.keys(renamedCategories).length > 0) {
+                const sorter = natsort();
                 categoryOrder.sort((a, b) => {
                     let val1 = categories[a];
                     let renamed1 = renamedCategories[val1];
@@ -264,23 +266,7 @@ class DotPlotCanvas extends React.PureComponent {
                     if (renamed2 != null) {
                         val2 = renamed2;
                     }
-                    val1 = ('' + val1).toLowerCase();
-                    val2 = ('' + val2).toLowerCase();
-                    const num1 = parseFloat(val1);
-                    const num1Valid = !isNaN(num1);
-                    const num2 = parseFloat(val2);
-                    const num2Valid = !isNaN(num2);
-
-                    if (num1Valid && num2Valid) {
-                        return (num1 === num2 ? 0 : (num1 < num2 ? -1 : 1));
-                    }
-                    if (num1Valid) {
-                        return 1;
-                    }
-                    if (num2Valid) {
-                        return -1;
-                    }
-                    return val1 === val2 ? 0 : (val1 > val2 ? 1 : -1);
+                    return sorter(val1, val2);
                 });
             }
 
@@ -292,8 +278,7 @@ class DotPlotCanvas extends React.PureComponent {
         this.features = features;
         this.size = this.getSize(context);
         this.categoryOrder = categoryOrder;
-        this.colorScale = scaleSequential(interpolateReds).domain(this.props.meanRange).clamp(true);
-        this.sizeScale = scaleLinear().domain(this.props.fractionRange).range([minRadius, maxRadius]).clamp(true);
+
     }
 
     handleSaveImageMenu = (event) => {
@@ -419,14 +404,6 @@ class DotPlotCanvas extends React.PureComponent {
             </Paper>
             <div ref={this.divRef}></div>
 
-            {this.props.legend && <ColorSchemeLegend style={{display: 'block', marginLeft: 10}}
-                                                     width={186}
-                                                     label={true} height={40}
-                                                     scale={this.colorScale}/>}
-            {this.props.legend && <SizeLegend style={{display: 'block'}}
-                                              width={150}
-                                              label={true} height={40}
-                                              scale={this.sizeScale}/>}
 
         </div>);
 
