@@ -1,4 +1,4 @@
-import {Switch} from '@material-ui/core';
+import {InputLabel, Switch} from '@material-ui/core';
 import MuiAccordionPanel from '@material-ui/core/Accordion';
 import MuiAccordionPanelDetails from '@material-ui/core/AccordionDetails';
 import MuiAccordionPanelSummary from '@material-ui/core/AccordionSummary';
@@ -12,7 +12,7 @@ import FormControlLabel from '@material-ui/core/FormControlLabel';
 import Grid from '@material-ui/core/Grid';
 import IconButton from '@material-ui/core/IconButton';
 import Input from '@material-ui/core/Input';
-import InputLabel from '@material-ui/core/InputLabel';
+
 import List from '@material-ui/core/List';
 import ListItem from '@material-ui/core/ListItem';
 import ListItemSecondaryAction from '@material-ui/core/ListItemSecondaryAction';
@@ -26,20 +26,17 @@ import CloudDownloadIcon from '@material-ui/icons/CloudDownload';
 import DeleteIcon from '@material-ui/icons/Delete';
 import HighlightOffIcon from '@material-ui/icons/HighlightOff';
 import SaveIcon from '@material-ui/icons/Save';
+import memoize from "memoize-one";
+import natsort from 'natsort';
 import React from 'react';
 import {connect} from 'react-redux';
 import {
     deleteDatasetFilter,
-    diffExp,
     downloadSelectedIds,
     exportDatasetFilters,
     getDatasetFilterArray,
-    getEmbeddingKey, getTraceKey,
-    handleBrushFilterUpdated,
-    handleCategoricalNameChange,
-    handleColorChange,
-    handleDimensionFilterUpdated,
-    handleMeasureFilterUpdated,
+    getEmbeddingKey,
+    getTraceKey,
     openDatasetFilter,
     removeDatasetFilter,
     SAVE_DATASET_FILTER_DIALOG,
@@ -53,19 +50,19 @@ import {
     setMarkerOpacityUI,
     setNumberOfBins,
     setNumberOfBinsUI,
-    setPointSize, setPrimaryTraceKey,
+    setPointSize,
+    setPrimaryTraceKey,
     setSearchTokens,
     setSelectedEmbedding,
     setUnselectedMarkerOpacity,
     setUnselectedMarkerOpacityUI
 } from './actions';
 import AutocompleteVirtualized from './AutocompleteVirtualized';
-import CategoricalLegend from './CategoricalLegend';
-import ColorSchemeLegendWrapper from './ColorSchemeLegendWrapper';
 import ColorSchemeSelector from './ColorSchemeSelector';
 import {intFormat} from './formatters';
 import {splitSearchTokens} from './util';
 
+const sorter = natsort();
 const pointSizeOptions = [{value: 0.25, label: '25%'}, {value: 0.5, label: '50%'}, {
     value: 0.75,
     label: '75%'
@@ -73,8 +70,8 @@ const pointSizeOptions = [{value: 0.25, label: '25%'}, {value: 0.5, label: '50%'
     value: 3,
     label: '300%'
 }, {value: 4, label: '400%'}];
-const gallerySizeOptions = [{value: 400, label: 'Small'}, {value: 500, label: 'Medium'}, {
-    value: 600,
+const gallerySizeOptions = [{value: 300, label: 'Small'}, {value: 500, label: 'Medium'}, {
+    value: 800,
     label: 'Large'
 }];
 const summaryOptions = [
@@ -141,6 +138,36 @@ const AccordionPanelDetails = withStyles(theme => ({
 }))(MuiAccordionPanelDetails);
 
 
+const getEmbeddingKeys = memoize(
+    (embeddings) => {
+        const embeddingKeys = embeddings.map(e => getEmbeddingKey(e));
+        embeddingKeys.sort(sorter);
+        return embeddingKeys;
+    }
+);
+
+const getAnnotationOptions = memoize(
+    (obs, obsCat) => {
+        let annotationOptions = obs.concat(obsCat);
+        annotationOptions.sort(sorter);
+        return annotationOptions;
+    }
+);
+const getFeatureSetOptions = memoize((markers) => {
+        const featureSetOptions = [];
+        for (const group in markers) {
+            const groupMarkers = markers[group];
+            for (const name in groupMarkers) {
+                featureSetOptions.push({group: group, text: name});
+            }
+        }
+        featureSetOptions.sort((item1, item2) => {
+            return sorter(item1.group, item2.group);
+        });
+        return featureSetOptions;
+    }
+);
+
 class EmbedForm extends React.PureComponent {
 
 
@@ -155,6 +182,7 @@ class EmbedForm extends React.PureComponent {
     onMarkerOpacityChange = (event) => {
         this.props.handleMarkerOpacityUI(event.target.value);
     };
+
     onPointSizeChange = (event) => {
         this.props.handlePointSize(event.target.value);
     };
@@ -179,7 +207,7 @@ class EmbedForm extends React.PureComponent {
                 break;
             }
         }
-    }
+    };
 
     onMarkerOpacityKeyPress = (event) => {
         if (event.key === 'Enter') {
@@ -223,7 +251,6 @@ class EmbedForm extends React.PureComponent {
         }
     };
 
-
     onChartSizeChange = (event) => {
         const value = event.target.value;
         this.props.handleChartSize(value);
@@ -242,14 +269,6 @@ class EmbedForm extends React.PureComponent {
         this.props.handleEmbeddings(embeddings.slice(0));
     };
 
-
-    handleReverseColors = (event) => {
-        const value = event.target.checked;
-        const interpolator = this.props.interpolator;
-        interpolator.reversed = value;
-        this.props.handleInterpolator(Object.assign({}, interpolator));
-    };
-
     handleBinValuesChange = (event) => {
         const value = event.target.checked;
         this.props.handleBinValues(value);
@@ -286,6 +305,7 @@ class EmbedForm extends React.PureComponent {
         event.preventDefault();
         this.props.downloadSelectedIds();
     };
+
     handleDiffExp = (event) => {
         event.preventDefault();
         this.props.handleDiffExp();
@@ -298,6 +318,7 @@ class EmbedForm extends React.PureComponent {
     onDatasetFilterCleared = () => {
         this.props.removeDatasetFilter(null);
     };
+
     onDatasetFilterSaved = () => {
         this.props.handleDialog(SAVE_DATASET_FILTER_DIALOG);
     };
@@ -306,13 +327,12 @@ class EmbedForm extends React.PureComponent {
         this.props.handleCombineDatasetFilters(event.target.checked ? 'or' : 'and');
     };
 
+
     render() {
         const {
-            categoricalNames, chartSize, numberOfBinsUI, interpolator, binValues, binSummary, embeddings, classes, embeddingData,
+            chartSize, numberOfBinsUI, binValues, binSummary, embeddings, classes,
             searchTokens, markerOpacity, datasetFilter, datasetFilters,
-            featureSummary, shape, nObsSelected, globalFeatureSummary, unselectedMarkerOpacity, dataset,
-            handleColorChange, handleNameChange, handleMeasureFilterUpdated, handleDimensionFilterUpdated, pointSize,
-            combineDatasetFilters, selection
+            unselectedMarkerOpacity, dataset, pointSize, combineDatasetFilters, selection, serverInfo
         } = this.props;
 
         let currentDatasetFilters = getDatasetFilterArray(datasetFilter);
@@ -336,19 +356,32 @@ class EmbedForm extends React.PureComponent {
         }
 
         // for filters we only need one embedding trace per feature
-        const traceNames = new Set();
-        const filterTraces = [];
-        embeddingData.forEach(trace => {
-            if (trace.active && trace.name !== '__count' && !traceNames.has(trace.name)) {
-                traceNames.add(trace.name);
-                filterTraces.push(trace);
-            }
-        });
-        filterTraces.sort((a, b) => {
-            a = a.name.toLowerCase();
-            b = b.name.toLowerCase();
-            return a < b ? -1 : (a === b ? 0 : 1);
-        });
+        // const traceNames = new Set();
+        // const filterTraces = [];
+        // let primaryTraceName;
+        // embeddingData.forEach(trace => {
+        //     if (primaryTraceKey === getTraceKey(trace)) {
+        //         primaryTraceName = trace.name;
+        //     }
+        //     if (trace.active && trace.name !== '__count' && !traceNames.has(trace.name)) {
+        //         traceNames.add(trace.name);
+        //         filterTraces.push(trace);
+        //     }
+        // });
+        // // put active trace 1st
+        // filterTraces.sort((a, b) => {
+        //     if (a.name === primaryTraceName) {
+        //         return -1;
+        //     }
+        //     if (b.name === primaryTraceName) {
+        //         return 1;
+        //     }
+        //     a = a.name.toLowerCase();
+        //     b = b.name.toLowerCase();
+        //     return a < b ? -1 : (a === b ? 0 : 1);
+        // });
+
+
         let savedDatasetFilter = this.props.savedDatasetFilter;
         if (savedDatasetFilter == null) {
             savedDatasetFilter = {};
@@ -356,46 +389,15 @@ class EmbedForm extends React.PureComponent {
         const splitTokens = splitSearchTokens(searchTokens);
         const featureOptions = dataset == null ? [] : dataset.features;
         const markers = dataset == null || dataset.markers == null ? {} : dataset.markers;
-        const featureSetOptions = [];
-
-        for (const group in markers) {
-            const groupMarkers = markers[group];
-            for (const name in groupMarkers) {
-                featureSetOptions.push({group: group, text: name});
-            }
-        }
-
-        featureSetOptions.sort((item1, item2) => {
-            let a = item1.group.toLowerCase();
-            let b = item2.group.toLowerCase();
-            if (a !== b) {
-                return a < b ? -1 : 1;
-            }
-            a = item1.text.toLowerCase();
-            b = item2.text.toLowerCase();
-            const aNumber = parseFloat(a);
-            const bNumber = parseFloat(b);
-            if (!isNaN(aNumber) && !isNaN(bNumber)) {
-                a = aNumber;
-                b = bNumber;
-            }
-            return a < b ? -1 : (a === b ? 0 : 1);
-        });
-
         const availableEmbeddings = dataset == null ? [] : dataset.embeddings;
-        const embeddingKeys = embeddings.map(e => getEmbeddingKey(e));
+
         const isSummarized = dataset == null ? false : dataset.precomputed != null;
         const obsCat = dataset == null ? [] : dataset.obsCat;
         const obs = dataset == null ? [] : dataset.obs;
-
-        let annotationOptions = obs.concat(obsCat);
-        annotationOptions.sort((a, b) => {
-            a = a.toLowerCase();
-            b = b.toLowerCase();
-            return a < b ? -1 : (a === b ? 0 : 1);
-        });
-
-
+        const embeddingKeys = getEmbeddingKeys(embeddings);
+        const annotationOptions = getAnnotationOptions(obs, obsCat);
+        const featureSetOptions = getFeatureSetOptions(markers);
+        const fancy = serverInfo.fancy;
         return (
             <div className={classes.root}>
                 <FormControl className={classes.formControl}>
@@ -425,7 +427,8 @@ class EmbedForm extends React.PureComponent {
                     {/*                    onChange={this.props.handleFeatures}*/}
                     {/*                    helperText={'Enter or paste list'}*/}
                     {/*                    isMulti={true}/>*/}
-                    <AutocompleteVirtualized  onChipClick={this.onFeatureClick} label={"Features"} options={featureOptions} value={splitTokens.X}
+                    <AutocompleteVirtualized onChipClick={this.onFeatureClick} label={"Features"}
+                                             options={featureOptions} value={splitTokens.X}
                                              onChange={this.onFeaturesChange}/>
                 </FormControl>
 
@@ -449,28 +452,97 @@ class EmbedForm extends React.PureComponent {
                                              onChange={this.onFeatureSetsChange} groupBy={true}/>
                 </FormControl>}
 
+
+                {/*<Accordion defaultExpanded>*/}
+                {/*    <AccordionPanelSummary*/}
+                {/*        aria-controls="summary-content"*/}
+                {/*        id="summary-header"*/}
+                {/*    >*/}
+                {/*        <div>Active List</div>*/}
+                {/*    </AccordionPanelSummary>*/}
+                {/*    <AccordionPanelDetails>*/}
+                {/*        <div style={{marginLeft: 10, maxHeight: 500, overflow: 'auto'}}>*/}
+                {/*            {filterTraces.map(traceInfo => {*/}
+
+                {/*                    return traceInfo.continuous ?*/}
+                {/*                        <ColorSchemeLegendWrapper*/}
+                {/*                            key={traceInfo.name}*/}
+                {/*                            handleDomain={this.props.handleDomain}*/}
+                {/*                            selected={traceInfo.name === primaryTraceName}*/}
+                {/*                            width={140}*/}
+                {/*                            showColorScheme={false}*/}
+                {/*                            height={30}*/}
+                {/*                            style={{*/}
+                {/*                                paddingBottom: 3,*/}
+                {/*                                paddingTop: 3,*/}
+                {/*                                display: 'block',*/}
+                {/*                                borderBottom: '1px solid rgba(0, 0, 0, 0.12)'*/}
+                {/*                            }}*/}
+                {/*                            handleUpdate={handleMeasureFilterUpdated}*/}
+                {/*                            datasetFilter={datasetFilter}*/}
+                {/*                            scale={traceInfo.colorScale}*/}
+                {/*                            featureSummary={featureSummary}*/}
+                {/*                            globalFeatureSummary={globalFeatureSummary}*/}
+                {/*                            nObs={shape[0]}*/}
+                {/*                            nObsSelected={nObsSelected}*/}
+                {/*                            maxHeight={null}*/}
+                {/*                            name={traceInfo.name}*/}
+                {/*                        /> :*/}
+                {/*                        <CategoricalLegend*/}
+                {/*                            selected={traceInfo.name === primaryTraceName}*/}
+                {/*                            key={traceInfo.name}*/}
+                {/*                            style={{*/}
+                {/*                                paddingBottom: 3,*/}
+                {/*                                paddingTop: 3,*/}
+                {/*                                display: 'block',*/}
+                {/*                                borderBottom: '1px solid rgba(0, 0, 0, 0.12)'*/}
+                {/*                            }}*/}
+                {/*                            datasetFilter={datasetFilter}*/}
+                {/*                            handleClick={handleDimensionFilterUpdated}*/}
+                {/*                            handleColorChange={handleColorChange}*/}
+                {/*                            handleNameChange={handleNameChange}*/}
+                {/*                            categoricalNames={categoricalNames}*/}
+                {/*                            name={traceInfo.name}*/}
+                {/*                            scale={traceInfo.colorScale}*/}
+                {/*                            maxHeight={300}*/}
+                {/*                            clickEnabled={true}*/}
+                {/*                            nObs={shape[0]}*/}
+                {/*                            nObsSelected={nObsSelected}*/}
+                {/*                            globalFeatureSummary={globalFeatureSummary}*/}
+                {/*                            featureSummary={featureSummary}/>;*/}
+                {/*                }*/}
+                {/*            )}*/}
+
+
+                {/*        </div>*/}
+                {/*    </AccordionPanelDetails>*/}
+                {/*</Accordion>*/}
+
                 <Accordion defaultExpanded>
                     <AccordionPanelSummary
-                        aria-controls="summary-content"
-                        id="summary-header"
+                        aria-controls="filter-content"
+                        id="filter-header"
                     >
-                        <div>Filter</div>
+                        <div>Filters</div>
                     </AccordionPanelSummary>
                     <AccordionPanelDetails>
-                        <div style={{marginLeft: 10}}>
-                            <div>
-                                <Grid component="label" container alignItems="center" spacing={0}>
-                                    <Grid item>AND</Grid>
-                                    <Grid item>
-                                        <Switch
-                                            size="small"
-                                            checked={combineDatasetFilters === 'or'}
-                                            onChange={this.handleCombineDatasetFilters}
-                                        />
-                                    </Grid>
-                                    <Grid item>OR</Grid>
+                        <div style={{marginLeft: 10, maxHeight: 500}}>
+
+                            <Grid component="label" alignContent={"flex-start"} container alignItems="center"
+                                  spacing={0}>
+                                <Grid item><InputLabel shrink={true} variant={"standard"}>Combine
+                                    Filters</InputLabel></Grid>
+                                <Grid item>AND</Grid>
+                                <Grid item>
+                                    <Switch
+                                        size="small"
+                                        checked={combineDatasetFilters === 'or'}
+                                        onChange={this.handleCombineDatasetFilters}
+                                    />
                                 </Grid>
-                            </div>
+                                <Grid item>OR</Grid>
+                            </Grid>
+
                             {datasetFilterKeys.length > 0 && !isNaN(selection.count) &&
                             <React.Fragment>
                                 <div style={{marginBottom: 2}}>
@@ -492,79 +564,23 @@ class EmbedForm extends React.PureComponent {
                                         <IconButton size={'small'} disabled={datasetFilterKeys.length === 0}
                                                     onClick={this.onDatasetFilterCleared}><HighlightOffIcon/></IconButton>
                                     </Tooltip>
-                                    <Tooltip title={"Save Filter"}>
-                                        <IconButton size={'small'}  disabled={datasetFilterKeys.length === 0}
+                                    {fancy && <Tooltip title={"Save Filter"}>
+                                        <IconButton size={'small'} disabled={datasetFilterKeys.length === 0}
                                                     onClick={this.onDatasetFilterSaved}><SaveIcon/></IconButton>
-                                    </Tooltip>
+                                    </Tooltip>}
                                     <Tooltip title={"Download Selected IDs"}>
-                                        <IconButton size={'small'}  disabled={datasetFilterKeys.length === 0}
+                                        <IconButton size={'small'} disabled={datasetFilterKeys.length === 0}
                                                     onClick={this.handleSelectedCellsClick}><CloudDownloadIcon/></IconButton>
                                     </Tooltip>
-                                    {/*<Tooltip title={"Compute Markers"}>*/}
-                                    {/*    <Button color="primary" variant="outlined"*/}
-                                    {/*            disabled={datasetFilterKeys.length === 0}*/}
-                                    {/*            size={"small"}*/}
-                                    {/*            onClick={this.handleDiffExp}>Markers</Button>*/}
-                                    {/*</Tooltip>*/}
                                     <Divider/>
                                 </div>
-
                             </React.Fragment>
                             }
 
 
-                            {filterTraces.map(traceInfo =>
-
-                                traceInfo.continuous ?
-                                    <ColorSchemeLegendWrapper
-                                        key={traceInfo.name}
-                                        width={140}
-                                        showColorScheme={false}
-                                        height={30}
-                                        style={{
-                                            paddingBottom: 3,
-                                            paddingTop: 3,
-                                            display: 'block',
-                                            borderBottom: '1px solid rgba(0, 0, 0, 0.12)'
-                                        }}
-                                        handleUpdate={handleMeasureFilterUpdated}
-                                        datasetFilter={datasetFilter}
-                                        scale={traceInfo.colorScale}
-                                        featureSummary={featureSummary}
-                                        globalFeatureSummary={globalFeatureSummary}
-                                        nObs={shape[0]}
-                                        nObsSelected={nObsSelected}
-                                        maxHeight={null}
-                                        name={traceInfo.name}
-                                    /> :
-                                    <CategoricalLegend
-                                        key={traceInfo.name}
-                                        style={{
-                                            paddingBottom: 3,
-                                            paddingTop: 3,
-                                            display: 'block',
-                                            borderBottom: '1px solid rgba(0, 0, 0, 0.12)'
-                                        }}
-                                        datasetFilter={datasetFilter}
-                                        handleClick={handleDimensionFilterUpdated}
-                                        handleColorChange={handleColorChange}
-                                        handleNameChange={handleNameChange}
-                                        categoricalNames={categoricalNames}
-                                        name={traceInfo.name}
-                                        scale={traceInfo.colorScale}
-                                        maxHeight={300}
-                                        clickEnabled={true}
-                                        nObs={shape[0]}
-                                        nObsSelected={nObsSelected}
-                                        globalFeatureSummary={globalFeatureSummary}
-                                        featureSummary={featureSummary}/>
-                            )}
-
-
                         </div>
                     </AccordionPanelDetails>
                 </Accordion>
-
                 <Accordion defaultExpanded>
                     <AccordionPanelSummary
                         aria-controls="view-options-content"
@@ -587,7 +603,7 @@ class EmbedForm extends React.PureComponent {
                                        className={classes.formControl} value={markerOpacity}/>
                             <TextField type="text"
                                        onKeyPress={this.onUnselectedMarkerOpacityKeyPress}
-                                       onChange={this.onUnselectedMarkerOpacityChange} label="Unselected Marker Opacity"
+                                       onChange={this.onUnselectedMarkerOpacityChange} label="Filtered Marker Opacity"
                                        className={classes.formControl} value={unselectedMarkerOpacity}/>
 
                             <FormControl className={classes.formControl}>
@@ -623,21 +639,11 @@ class EmbedForm extends React.PureComponent {
                             </FormControl>
 
                             <FormControl className={classes.formControl}>
-                                <InputLabel htmlFor="color-scheme">Continuous Color Scale</InputLabel>
+                                <InputLabel htmlFor="color-scheme">Color Scale</InputLabel>
                                 <ColorSchemeSelector/>
                             </FormControl>
-                            <div><FormControlLabel
-                                control={
-                                    <Switch
-                                        checked={interpolator.reversed || false}
-                                        value={'reverseColors'}
-                                        onChange={this.handleReverseColors}
-                                    />
-                                }
-                                label="Reverse Colors"
-                            /></div>
 
-                            {!isSummarized && <div><FormControlLabel
+                            {!isSummarized && fancy && <div><FormControlLabel
                                 control={
                                     <Switch
                                         checked={binValues}
@@ -687,7 +693,7 @@ class EmbedForm extends React.PureComponent {
 
                     </AccordionPanelDetails>
                 </Accordion>
-                <Accordion defaultExpanded>
+                {fancy && <Accordion defaultExpanded>
                     <AccordionPanelSummary
                         aria-controls="filter-options-content"
                         id="filter-options-header"
@@ -716,7 +722,7 @@ class EmbedForm extends React.PureComponent {
                             </div>}
                         </div>
                     </AccordionPanelDetails>
-                </Accordion>
+                </Accordion>}
             </div>
         );
     }
@@ -724,11 +730,6 @@ class EmbedForm extends React.PureComponent {
 
 const mapStateToProps = state => {
     return {
-        categoricalNames: state.categoricalNames,
-        featureSummary: state.featureSummary,
-        shape: state.dataset.shape,
-        nObsSelected: state.selection.count,
-        globalFeatureSummary: state.globalFeatureSummary,
         dataset: state.dataset,
         binValues: state.binValues,
         binSummary: state.binSummary,
@@ -739,7 +740,9 @@ const mapStateToProps = state => {
         interpolator: state.interpolator,
         markerOpacity: state.markerOpacityUI,
         pointSize: state.pointSize,
+        primaryTraceKey: state.primaryTraceKey,
         savedDatasetFilter: state.savedDatasetFilter,
+        serverInfo: state.serverInfo,
         embeddings: state.embeddings,
         searchTokens: state.searchTokens,
         unselectedMarkerOpacity: state.unselectedMarkerOpacityUI,
@@ -771,24 +774,6 @@ const mapDispatchToProps = (dispatch, ownProps) => {
         removeDatasetFilter: (filter) => {
             dispatch(removeDatasetFilter(filter));
         },
-        handleDimensionFilterUpdated: (e) => {
-            dispatch(handleDimensionFilterUpdated(e));
-        },
-        handleColorChange: (e) => {
-            dispatch(handleColorChange(e));
-        },
-        handleNameChange: (e) => {
-            dispatch(handleCategoricalNameChange(e));
-        },
-        handleMeasureFilterUpdated: (e) => {
-            dispatch(handleMeasureFilterUpdated(e));
-        },
-        onSelect: (e) => {
-            dispatch(handleBrushFilterUpdated(e));
-        },
-        onDeselect: (e) => {
-            dispatch(handleBrushFilterUpdated(e));
-        },
         handleEmbeddings: value => {
             dispatch(setSelectedEmbedding(value));
         },
@@ -833,7 +818,8 @@ const mapDispatchToProps = (dispatch, ownProps) => {
         },
         handleExportDatasetFilters: () => {
             dispatch(exportDatasetFilters());
-        }
+        },
+
     };
 };
 
