@@ -1,16 +1,33 @@
 import React from 'react';
 
 import {connect} from 'react-redux';
-import {getEmbeddingKey, getTraceKey, MORE_OPTIONS_DIALOG, setDialog} from './actions';
+import {getEmbeddingKey, getTraceKey, setPrimaryChartSize} from './actions';
 import EmbeddingChart from './EmbeddingChart';
-import {getChartSize} from './util';
+
 
 const emptySet = new Set();
 
 class EmbeddingCharts extends React.PureComponent {
 
+    constructor(props) {
+        super(props);
+        this.resizeListener = () => {
+            let width = window.innerWidth - 280;
+            // let height = Math.max(1, this.containerElementRef.offsetHeight);
+            let height = Math.max(300, window.innerHeight - 220);
+            this.props.handlePrimaryChartSize({width: width, height: height});
+        };
+
+        window.addEventListener('resize', this.resizeListener);
+    }
+
+    componentWillUnmount() {
+        window.removeEventListener('resize', this.resizeListener);
+    }
+
+
     render() {
-        const {primaryTraceKey, embeddingData, markerOpacity, unselectedMarkerOpacity, selection} = this.props;
+        const {primaryTraceKey, embeddingData, selection, onGallery, primaryChartSize} = this.props;
         let primaryTraces = embeddingData.filter(traceInfo => getTraceKey(traceInfo) === primaryTraceKey);
         const primaryTrace = primaryTraces.length === 1 ? primaryTraces[0] : null;
         let userPoints = emptySet;
@@ -22,17 +39,13 @@ class EmbeddingCharts extends React.PureComponent {
         }
 
         if (primaryTrace == null) {
-            const chartSize = getChartSize();
-            return <div style={{height: chartSize.height}}></div>;
+            return <div style={{height: primaryChartSize.height}}></div>;
         }
+
         return (<EmbeddingChart
-                markerOpacity={markerOpacity}
-                unselectedMarkerOpacity={unselectedMarkerOpacity}
+                onGallery={onGallery}
                 traceInfo={primaryTrace}
                 selection={userPoints}
-                color={primaryTrace.colors}
-                onMoreOptions={this.props.handleMoreOptions}
-                onGallery={this.props.onGallery}
             />
         );
     }
@@ -42,16 +55,15 @@ class EmbeddingCharts extends React.PureComponent {
 const mapStateToProps = state => {
     return {
         embeddingData: state.embeddingData,
-        markerOpacity: state.markerOpacity,
-        unselectedMarkerOpacity: state.unselectedMarkerOpacity,
-        selection: state.selection,
-        primaryTraceKey: state.primaryTraceKey
+        primaryTraceKey: state.primaryTraceKey,
+        primaryChartSize: state.primaryChartSize,
+        selection: state.selection
     };
 };
 const mapDispatchToProps = dispatch => {
     return {
-        handleMoreOptions: () => {
-            dispatch(setDialog(MORE_OPTIONS_DIALOG));
+        handlePrimaryChartSize: value => {
+            dispatch(setPrimaryChartSize(value));
         }
     };
 };
