@@ -9,18 +9,21 @@ import {getTraceKey, setEmbeddingData, setPrimaryTraceKey} from './actions';
 import GalleryImage from './GalleryImage';
 import {createScatterPlot} from './ThreeUtil';
 
+function createContainer(chartSize) {
+    const containerElement = document.createElement('div');
+    containerElement.style.position = 'absolute';
+    containerElement.style.left = '-9999px';
+    containerElement.style.width = chartSize + 'px';
+    containerElement.style.height = chartSize + 'px';
+    return containerElement;
+}
+
 class GalleryCharts extends React.PureComponent {
     constructor(props) {
         super(props);
-        const containerElement = document.createElement('div');
-        containerElement.style.position = 'absolute';
-        containerElement.style.left = '-9999px';
-        containerElement.style.width = props.chartSize + 'px';
-        containerElement.style.height = props.chartSize + 'px';
-        document.body.appendChild(containerElement);
-        this.scatterPlot = createScatterPlot(containerElement, window.ApplePaySession);
-        this.scatterPlot.interactive = false;
-        this.containerElement = containerElement;
+        this.containerElement = createContainer(this.props.chartSize);
+        document.body.appendChild(this.containerElement);
+        this.scatterPlot = createScatterPlot(this.containerElement, window.ApplePaySession, false, false);
         this.emptySet = new Set();
 
     }
@@ -36,16 +39,15 @@ class GalleryCharts extends React.PureComponent {
         this.props.handleEmbeddingData(this.props.embeddingData.slice());
     };
 
-    componentDidUpdate(prevProps, prevState, snapshot) {
-        if (prevProps.chartSize !== this.props.chartSize) {
-            this.containerElement.style.width = this.props.chartSize + 'px';
-            this.containerElement.style.height = this.props.chartSize + 'px';
-            this.scatterPlot.resize(false);
-        }
-    }
 
     render() {
-        const {chartSize, embeddingData, markerOpacity, unselectedMarkerOpacity, pointSize, chartOptions, selection} = this.props;
+        let {categoricalNames, chartSize, embeddingData, primaryChartSize, markerOpacity, unselectedMarkerOpacity, pointSize, chartOptions, selection} = this.props;
+        if (this.containerElement.style.width !== this.props.chartSize + 'px') {
+            document.body.removeChild(this.containerElement);
+            this.containerElement = createContainer(this.props.chartSize);
+            document.body.appendChild(this.containerElement);
+            this.scatterPlot = createScatterPlot(this.containerElement, window.ApplePaySession, false, false);
+        }
         let galleryTraces = embeddingData.filter(traceInfo => traceInfo.active);
 
         for (let i = 0; i < galleryTraces.length; i++) {
@@ -65,6 +67,8 @@ class GalleryCharts extends React.PureComponent {
             chartOptions={chartOptions}
             pointSize={pointSize}
             chartSize={chartSize}
+            categoricalNames={categoricalNames}
+            primaryChartSize={primaryChartSize}
             unselectedMarkerOpacity={unselectedMarkerOpacity}
             selection={selection}
             containerElement={this.containerElement}
@@ -94,6 +98,7 @@ class GalleryCharts extends React.PureComponent {
 
 const mapStateToProps = state => {
     return {
+        categoricalNames: state.categoricalNames,
         embeddingData: state.embeddingData,
         markerOpacity: state.markerOpacity,
         unselectedMarkerOpacity: state.unselectedMarkerOpacity,
@@ -101,6 +106,7 @@ const mapStateToProps = state => {
         primaryTraceKey: state.primaryTraceKey,
         pointSize: state.pointSize,
         chartSize: state.chartSize,
+        primaryChartSize: state.primaryChartSize,
         chartOptions: state.chartOptions
     };
 };
