@@ -11,10 +11,14 @@ import {saveImage} from './ChartUtil';
 import {numberFormat} from './formatters';
 import OpenseadragonSvgOverlay from './OpenseadragonSvgOverlay';
 import {getCategoryLabelsPositions} from './ThreeUtil';
-import {arrayToSvgPath, getChartSize, isPointInside} from './util';
+import {arrayToSvgPath, isPointInside} from './util';
 
 
-export function drawImage(context, chartSize, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, showLabels, categoricalNames) {
+export function getSpotRadius(trace, pointSize) {
+    return pointSize * (trace.embedding.spatial.spot_diameter ? trace.embedding.spatial.spot_diameter / 2 : 20);
+}
+
+export function drawImage(context, chartSize, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, chartOptions, categoricalNames, spotRadius) {
     if (traceInfo.tileSource.ready) {
         const img = traceInfo.tileSource.levels[traceInfo.tileSource.levels.length - 1].context2D.canvas;
         if (chartSize == null) {
@@ -23,43 +27,79 @@ export function drawImage(context, chartSize, traceInfo, selection, markerOpacit
         const zoom = Math.min(chartSize.width / img.width, chartSize.height / img.height);
         context.drawImage(img, 0, 0, img.width * zoom, img.height * zoom);
         context.scale(zoom, zoom);
-        drawSpots(context, zoom, traceInfo, selection, markerOpacity, unselectedMarkerOpacity);
-        drawLabels(context, zoom, traceInfo, showLabels, categoricalNames);
+        drawSpots(context, zoom, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, spotRadius);
+        drawLabels(context, zoom, traceInfo, chartOptions, categoricalNames);
         context.setTransform(1, 0, 0, 1, 0, 0);
     }
 }
 
-function drawLabels(context, zoom, traceInfo, showLabels, categoricalNames) {
-    showLabels = showLabels && traceInfo.isCategorical;
+function drawLabels(context, zoom, traceInfo, chartOptions, categoricalNames) {
+    const showLabels = chartOptions.showLabels && traceInfo.isCategorical;
     if (showLabels) {
-        context.fillStyle = 'black';
-        const fontSize = Math.ceil(this.props.chartOptions.labelFontSize * 1 / zoom);
-        context.font = fontSize + 'px Roboto Condensed,Helvetica,Arial,sans-serif';
         context.textAlign = 'center';
+        context.textBaseline = "middle";
+        const darkMode = true;
+        const fontSize = Math.ceil(chartOptions.labelFontSize / zoom);
+        context.fillStyle = darkMode ? 'white' : 'black';
+        context.strokeStyle = darkMode ? 'rgba(0,0,0,0.9)' : 'rgba(255,255,255,0.9)';
+        context.lineWidth = chartOptions.labelStrokeWidth;
+        context.font = fontSize + 'px Roboto Condensed,Helvetica,Arial,sans-serif';
         const labelsPositions = getCategoryLabelsPositions(traceInfo, categoricalNames);
         for (let i = 0, index = 0, n = labelsPositions.labels.length; i < n; i++, index += 3) {
             let x = labelsPositions.positions[index];
             let y = labelsPositions.positions[index + 1];
+            context.strokeText('' + labelsPositions.labels[i], x, y);
             context.fillText('' + labelsPositions.labels[i], x, y);
         }
     }
 }
 
-function drawSpots(context, zoom, traceInfo, selection, markerOpacity, unselectedMarkerOpacity) {
+function drawSpots(context, zoom, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, spotRadius) {
     context.lineWidth = 2 * 1 / zoom;
-    let spotRadius = traceInfo.embedding.spatial.spot_diameter / 2;
     if (context.setLineDash) {
         context.setLineDash([2, 2]);
     }
-    for (let i = 0; i < traceInfo.x.length; i++) {
-        let x = traceInfo.x[i];
-        let y = traceInfo.y[i];
-        const isSelected = selection.size === 0 || selection.has(i);
-        context.globalAlpha = isSelected ? markerOpacity : unselectedMarkerOpacity;
-        context.fillStyle = traceInfo.colors[i];
-        context.beginPath();
-        context.arc(x, y, spotRadius, 0, Math.PI * 2, true);
-        context.fill();
+    const isSelectionEmpty = selection.size === 0;
+    const indices = traceInfo.indices;
+    if (!isSelectionEmpty) { // draw unselected cells 1st
+        context.globalAlpha = unselectedMarkerOpacity;
+
+        for (let i = 0; i < traceInfo.x.length; i++) {
+            let index = indices[i];
+            let x = traceInfo.x[index];
+            let y = traceInfo.y[index];
+            const isSelected = selection.has(index);
+            if (!isSelected) {
+                context.fillStyle = traceInfo.colors[index];
+                context.beginPath();
+                context.arc(x, y, spotRadius, 0, Math.PI * 2, true);
+                context.fill();
+            }
+        }
+        context.globalAlpha = markerOpacity;
+        for (let i = 0; i < traceInfo.x.length; i++) {
+            let index = indices[i];
+            let x = traceInfo.x[index];
+            let y = traceInfo.y[index];
+            const isSelected = selection.has(index);
+            if (isSelected) {
+                context.fillStyle = traceInfo.colors[index];
+                context.beginPath();
+                context.arc(x, y, spotRadius, 0, Math.PI * 2, true);
+                context.fill();
+            }
+        }
+    } else {
+        context.globalAlpha = markerOpacity;
+        for (let i = 0; i < traceInfo.x.length; i++) {
+            let index = indices[i];
+            let x = traceInfo.x[index];
+            let y = traceInfo.y[index];
+            context.fillStyle = traceInfo.colors[index];
+            context.beginPath();
+            context.arc(x, y, spotRadius, 0, Math.PI * 2, true);
+            context.fill();
+        }
     }
     context.globalAlpha = 1;
     if (context.setLineDash) {
@@ -91,9 +131,7 @@ class ImageChart extends React.PureComponent {
     constructor(props) {
         super(props);
         this.id = uniqueId('cirro-image');
-        this.chartSize = getChartSize();
         this.tooltipElementRef = React.createRef();
-        this.props.chartOptions = {dragmode: 'pan', editSelection: false, showLabels: false};
     }
 
     findPointsInPolygon(points) {
@@ -110,7 +148,7 @@ class ImageChart extends React.PureComponent {
 
     findPointsInRectangle(rect) {
         let data = this.props.traceInfo;
-        const spotRadius = data.embedding.spatial.spot_diameter / 2;
+        const spotRadius = getSpotRadius(data, this.props.pointSize);
         let indices = [];
         const x = parseFloat(rect.getAttribute('x'));
         const y = parseFloat(rect.getAttribute('y'));
@@ -126,9 +164,10 @@ class ImageChart extends React.PureComponent {
         return indices;
     }
 
+
     findPointIndex(xpix, ypix) {
         let data = this.props.traceInfo;
-        let spotRadius = data.embedding.spatial.spot_diameter / 2;
+        const spotRadius = getSpotRadius(data, this.props.pointSize);
         for (let i = 0; i < data.x.length; i++) {
             if (Math.abs(data.x[i] - xpix) <= spotRadius && Math.abs(data.y[i] - ypix) <= spotRadius) {
                 return i;
@@ -173,12 +212,11 @@ class ImageChart extends React.PureComponent {
         let context = opts.context;
         let traceInfo = this.props.traceInfo;
         const selection = this.props.selection;
-
         let markerOpacity = this.props.markerOpacity;
         let unselectedMarkerOpacity = this.props.unselectedMarkerOpacity;
-        drawSpots(context, opts.zoom, traceInfo, selection, markerOpacity, unselectedMarkerOpacity);
-        drawLabels(context, opts.zoom, traceInfo, this.props.chartOptions.showLabels, this.props.categoricalNames);
-
+        const spotRadius = getSpotRadius(traceInfo, this.props.pointSize);
+        drawSpots(context, opts.zoom, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, spotRadius);
+        drawLabels(context, opts.zoom, traceInfo, this.props.chartOptions, this.props.categoricalNames);
     }
 
     createViewer() {
@@ -198,7 +236,7 @@ class ImageChart extends React.PureComponent {
         this.viewer = new OpenSeadragon({
             id: this.id,
             gestureSettingsMouse: {dblClickToZoom: true, clickToZoom: false},
-            autoResize: false,
+            autoResize: true,
             showFullPageControl: false,
             collectionMode: false,
             // visibilityRatio: 0.2,
@@ -368,14 +406,15 @@ class ImageChart extends React.PureComponent {
             elem.setAttribute('stroke-dasharray', '2 2');
             svgOverlay.node().appendChild(elem);
         });
-
-
     }
 
     componentDidUpdate(prevProps, prevState, snapshot) {
         if (this.canvasOverlay) {
             this.canvasOverlay._updateCanvas();
         }
+        // if (this.viewer && prevProps.chartSize !== this.props.chartSize) {
+        //     this.viewer.viewport.resize();
+        // }
     }
 
     componentWillUnmount() {
@@ -409,10 +448,6 @@ class ImageChart extends React.PureComponent {
         this.props.setChartOptions(this.props.chartOptions);
     };
 
-    onGallery = () => {
-        this.props.onGallery();
-    };
-
     onZoomIn = () => {
         this.viewer.viewport.zoomBy(this.viewer.zoomPerClick / 1.0);
         this.viewer.viewport.applyConstraints();
@@ -435,24 +470,24 @@ class ImageChart extends React.PureComponent {
 
 
     render() {
-
         return <React.Fragment>
             <div className={this.props.classes.root}>
                 <ChartToolbar
                     dragmode={this.props.chartOptions.dragmode}
-                    animating={false}
                     editSelection={this.props.chartOptions.editSelection}
+                    showLabels={this.props.chartOptions.showLabels}
+                    onMoreOptions={this.props.onMoreOptions}
+                    onGallery={this.props.onGallery}
+                    animating={false}
                     onZoomIn={this.onZoomIn}
                     onZoomOut={this.onZoomOut}
-                    onMoreOptions={this.onMoreOptions}
-                    showLabels={this.props.chartOptions.showLabels}
                     is3d={false}
                     onHome={this.onHome}
                     onSaveImage={this.onSaveImage}
                     onShowLabels={this.onShowLabels}
                     onDragMode={this.onDragMode}
                     onEditSelection={this.onEditSelection}
-                    onGallery={this.onGallery}>
+                >
                 </ChartToolbar>
                 <Typography color="textPrimary" ref={this.tooltipElementRef} style={{
                     display: 'inline-block',
@@ -463,8 +498,8 @@ class ImageChart extends React.PureComponent {
 
             <div style={{
                 display: 'inline-block',
-                width: this.chartSize.width,
-                height: this.chartSize.height
+                width: this.props.chartSize.width,
+                height: this.props.chartSize.height
             }}
                  id={this.id}>
             </div>
@@ -473,4 +508,4 @@ class ImageChart extends React.PureComponent {
 }
 
 
-export default withStyles(styles)(ImageChart);
\ No newline at end of file
+export default withStyles(styles)(ImageChart);
