@@ -9,7 +9,7 @@ import ChartToolbar from './ChartToolbar';
 import {saveImage} from './ChartUtil';
 import {numberFormat} from './formatters';
 import {createScatterPlot, getCategoryLabelsPositions, POINT_VISUALIZER_ID, updateScatterChart} from './ThreeUtil';
-import {getChartSize, isPointInside} from './util';
+import {isPointInside} from './util';
 
 function clamp(x, min_v, max_v) {
     return Math.min(Math.max(x, min_v), max_v);
@@ -67,21 +67,18 @@ class ScatterChartThree extends React.PureComponent {
         this.containerElementRef = React.createRef();
         this.tooltipElementRef = React.createRef();
         this.scatterPlot = null;
-        this.chartSize = getChartSize();
         this.lastHoverIndex = -1;
-        // window.addEventListener('resize', () => {
-        //     scatterGL.resize();
-        // });
     }
 
-
     componentDidUpdate(prevProps, prevState, snapshot) {
+        if (prevProps.chartSize !== this.props.chartSize) {
+            this.scatterPlot.resize();
+        }
         this.draw();
     }
 
     componentDidMount() {
         this.init();
-
         this.draw(); // TODO fix 2x draw
         this.draw();
         if (this.props.chartOptions.camera) {
@@ -159,8 +156,9 @@ class ScatterChartThree extends React.PureComponent {
         modelViewMatrix.multiplyMatrices(camera.matrixWorldInverse, object.matrixWorld);
         let gl_PointSize = (outputPointSize * scaleFactor) / 4;
         const showFog = chartOptions.showFog;
+        const isSelectionEmpty = selection.size === 0;
         for (let i = 0, j = 0, k = 0; i < npoints; i++, j += 4, k += 3) {
-            const isSelected = selection.size === 0 || selection.has(i);
+            const isSelected = isSelectionEmpty || selection.has(i);
             pos.x = positions[k];
             pos.y = positions[k + 1];
             pos.z = positions[k + 2];
@@ -239,8 +237,8 @@ class ScatterChartThree extends React.PureComponent {
 
 
     onSaveImage = (format) => {
-        const {traceInfo} = this.props;
-        saveImage(traceInfo, this.chartSize, bind(this.drawContext, this), format);
+        const {traceInfo, chartSize} = this.props;
+        saveImage(traceInfo, chartSize, bind(this.drawContext, this), format);
     };
 
     onMoreOptions = () => {
@@ -337,8 +335,8 @@ class ScatterChartThree extends React.PureComponent {
                     const traceInfo = this.props.traceInfo;
                     const positions = traceInfo.positions;
                     const camera = this.scatterPlot.camera;
-                    const widthHalf = this.chartSize.width / 2;
-                    const heightHalf = this.chartSize.height / 2;
+                    const widthHalf = this.props.chartSize.width / 2;
+                    const heightHalf = this.props.chartSize.height / 2;
                     const pos = new Vector3();
                     let selectedIndex = -1;
                     const tolerance = 2;
@@ -394,8 +392,8 @@ class ScatterChartThree extends React.PureComponent {
                 const traceInfo = this.props.traceInfo;
                 const positions = traceInfo.positions;
                 const camera = this.scatterPlot.camera;
-                const widthHalf = this.chartSize.width / 2;
-                const heightHalf = this.chartSize.height / 2;
+                const widthHalf = this.props.chartSize.width / 2;
+                const heightHalf = this.props.chartSize.height / 2;
                 const pos = new Vector3();
                 const selectedPoints = [];
 
@@ -428,8 +426,8 @@ class ScatterChartThree extends React.PureComponent {
                 const traceInfo = this.props.traceInfo;
                 const positions = traceInfo.positions;
                 const camera = this.scatterPlot.camera;
-                const widthHalf = this.chartSize.width / 2;
-                const heightHalf = this.chartSize.height / 2;
+                const widthHalf = this.props.chartSize.width / 2;
+                const heightHalf = this.props.chartSize.height / 2;
                 const pos = new Vector3();
                 const selectedPoints = [];
 
@@ -502,8 +500,8 @@ class ScatterChartThree extends React.PureComponent {
 
             <div style={{
                 display: 'inline-block',
-                width: this.chartSize.width,
-                height: this.chartSize.height
+                width: this.props.chartSize.width,
+                height: this.props.chartSize.height
             }}
                  ref={this.containerElementRef}>
             </div>
