@@ -17,6 +17,7 @@ import {
     SET_DATASET_FILTER,
     SET_DATASET_FILTERS,
     SET_DIALOG,
+    SET_DOMAIN,
     SET_DOT_PLOT_DATA,
     SET_DOT_PLOT_SORT_ORDER,
     SET_EMAIL,
@@ -32,6 +33,7 @@ import {
     SET_NUMBER_OF_BINS,
     SET_NUMBER_OF_BINS_UI,
     SET_POINT_SIZE,
+    SET_PRIMARY_CHART_SIZE,
     SET_PRIMARY_TRACE_KEY,
     SET_SAVED_DATASET_STATE,
     SET_SEARCH_TOKENS,
@@ -56,17 +58,21 @@ export const DEFAULT_INTERPOLATOR = 'Viridis';
 export const DEFAULT_DRAG_MODE = 'pan';
 export const DEFAULT_SHOW_LABELS = false;
 export const DEFAULT_SHOW_AXIS = true;
-export const DEFAULT_SHOW_FOG = true;
+export const DEFAULT_SHOW_FOG = false;
 export const DEFAULT_DARK_MODE = false;
 export const DEFAULT_LABEL_FONT_SIZE = 14;
 export const DEFAULT_LABEL_STROKE_WIDTH = 4;
 
 const DEFAULT_INTERPOLATOR_OBJ = {
     name: DEFAULT_INTERPOLATOR,
-    reversed: false,
     value: getInterpolator(DEFAULT_INTERPOLATOR)
 };
 
+const DEFAULT_PRIMARY_CHART_SIZE = {
+    width: window.innerWidth - 280,
+    height: Math.max(300, window.innerHeight - 220)
+};
+
 const DEFAULT_CHART_OPTIONS = {
     animating: false,
     dragmode: DEFAULT_DRAG_MODE,
@@ -88,6 +94,16 @@ function chartSize(state = 500, action) {
     }
 }
 
+
+function primaryChartSize(state = DEFAULT_PRIMARY_CHART_SIZE, action) {
+    switch (action.type) {
+        case SET_PRIMARY_CHART_SIZE:
+            return action.payload;
+        default:
+            return state;
+    }
+}
+
 /**
  *
  * @param state Array of value, type where type can be X, obs, or obsCat
@@ -496,6 +512,8 @@ function selectedDotPlotData(state = [], action) {
 
 function datasetFilters(state = [], action) {
     switch (action.type) {
+        case SET_DATASET:
+            return [];
         case SET_DATASET_FILTERS:
             //action.payload.sort()
             return action.payload;
@@ -507,6 +525,8 @@ function datasetFilters(state = [], action) {
 
 function datasetFilter(state = {}, action) {
     switch (action.type) {
+        case SET_DATASET:
+            return {};
         case SET_DATASET_FILTER:
             return action.payload;
         case RESTORE_VIEW:
@@ -520,6 +540,8 @@ function combineDatasetFilters(state = 'and', action) {
     switch (action.type) {
         case SET_COMBINE_DATASET_FILTERS:
             return action.payload;
+        case SET_DATASET:
+            return "and";
         case RESTORE_VIEW:
             return action.payload.combineDatasetFilters != null ? action.payload.combineDatasetFilters : state;
         default:
@@ -557,6 +579,22 @@ function embeddingData(state = [], action) {
             return action.payload;
         case SET_SELECTION:
             return state.slice();
+        case SET_DOMAIN:
+            state.forEach((traceInfo, stateIndex) => {
+                if (traceInfo.continuous && traceInfo.name === action.payload.name) {
+                    const summary = action.payload.summary;
+                    let domain = [summary.min, summary.max];
+                    if (summary.customMin != null && !isNaN(summary.customMin)) {
+                        domain[0] = summary.customMin;
+                    }
+                    if (summary.customMax != null && !isNaN(summary.customMax)) {
+                        domain[1] = summary.customMax;
+                    }
+                    traceInfo.colorScale.domain(domain);
+                    updateTraceColors(traceInfo);
+                }
+            });
+            return state.slice();
         case SET_CATEGORICAL_COLOR:
             state.forEach((traceInfo, stateIndex) => {
                 if (!traceInfo.continuous && traceInfo.name === action.payload.name) {
@@ -575,13 +613,6 @@ function embeddingData(state = [], action) {
             state.forEach((traceInfo, stateIndex) => {
                 if (traceInfo.continuous) {
                     let domain = traceInfo.colorScale.domain();
-                    // ensure domain is in order min to max if not reversed, otherwise max to min
-                    if (action.payload.reversed) {
-                        domain.sort((a, b) => b - a);
-                    } else {
-                        domain.sort((a, b) => a - b);
-                    }
-
                     traceInfo.colorScale = scaleSequential(action.payload.value).domain(domain);
                     updateTraceColors(traceInfo);
                 }
@@ -616,6 +647,8 @@ export function savedDatasetState(state = {}, action) {
 
 export function primaryTraceKey(state = null, action) {
     switch (action.type) {
+        case SET_DATASET:
+            return null;
         case SET_PRIMARY_TRACE_KEY:
             return action.payload;
         case SET_EMBEDDING_DATA:
@@ -677,6 +710,7 @@ export default combineReducers({
     markerOpacityUI,
     numberOfBins,
     numberOfBinsUI,
+    primaryChartSize,
     message,
     pointSize,
     primaryTraceKey,
